﻿using System.Collections.ObjectModel;
using System.Windows;
using System.Linq;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;

namespace BlotterSpike
{
    public partial class TheXamDataGridBlotter
    {
        public ObservableCollection<DataModel> tradeData;

        public TheXamDataGridBlotter()
        {
            InitializeComponent();

            tradeData = new ObservableCollection<DataModel>();
            var trade1 = new DataModel
            {
                NearRef = "ABC",
                NearAmount1 = 10,
                NearSubRootId = "1",
                FarRef = "DEF",
                FarAmount1 = 10,
                FarSubRootId = "1",
                NearName = "NearNameABC",
                FarName = "FarNameABC",
                NearCurrency1 = "GBP",
                FarCurrency1 = "EUR"
            };
            var trade2 = new DataModel
            {
                NearRef = "GHI",
                NearAmount1 = 20,
                NearSubRootId = "2",
                FarRef = "JKL",
                FarAmount1 = 10,
                FarSubRootId = "2",
                NearName = "NearNameGHI",
                FarName = "FarNameGHI",
                NearCurrency1 = "USD",
                FarCurrency1 = "JPY"
            };
            var trade3 = new DataModel
            {
                NearRef = "MNO",
                NearAmount1 = 20,
                NearSubRootId = "3",
                NearName = "NearNameMNO",
                NearCurrency1 = "CZK"
            };
            var trade4 = new DataModel
            {
                NearRef = "PQR",
                NearAmount1 = 30,
                NearSubRootId = "4",
                FarRef = "STU",
                FarAmount1 = 10,
                FarSubRootId = "4",
                NearName = "NearNamePQR",
                FarName = "FarNamePQR",
                NearCurrency1 = "CHF",
                FarCurrency1 = "NZD"
            };

            tradeData.Add(trade1);
            tradeData.Add(trade2);
            tradeData.Add(trade3);
            tradeData.Add(trade4);

            XamDataGrid1.DataSource = tradeData;
        }

        private void TheXamDataGridBlotter_OnLoaded(object sender, RoutedEventArgs e)
        {
            //to implement
        }

        private void XamDataGrid1_OnFieldPositionChanged(object sender, FieldPositionChangedEventArgs e)
        {
            //move the linked field
            //we want to move the linked cell
            var linkedName = e.Field.Name.Contains("Near")
                ? e.Field.Name.Replace("Near", "Far")
                : e.Field.Name.Replace("Far", "Near");

            //get the linked field
            var linkedField = XamDataGrid1.FieldLayouts[0].Fields[linkedName];
            if (linkedField != null)
            {
                var rowNumber = linkedField.Name.Contains("Near") ? 0 : 1;
                linkedField.ActualPosition = new FieldPosition(e.Field.ActualPosition.Column, rowNumber,
                    e.Field.ActualPosition.ColumnSpan, e.Field.ActualPosition.RowSpan);

                var metadata = e.Field.Tag as FieldMetaData;
                if (metadata.OldColumn < e.Field.ActualPosition.Column) // column was moved to the right
                {
                    var lastField = linkedField;

                    // this loop will shift overlapping columns to the left until there are no more overlapping columns
                    for (int i = e.Field.ActualPosition.Column; i >= 0; i--)
                    {
                        var overlappingFields = XamDataGrid1.FieldLayouts[0].Fields.Where(x => x.ActualPosition.Row == linkedField.ActualPosition.Row && x.ActualPosition.Column == i);
                        if (overlappingFields.Count() > 1)
                        {
                            var overlappedField = overlappingFields.Where(x => x != lastField).FirstOrDefault();
                            overlappedField.ActualPosition = new FieldPosition(overlappedField.ActualPosition.Column - 1, overlappedField.ActualPosition.Row,
                                overlappedField.ActualPosition.ColumnSpan, overlappedField.ActualPosition.RowSpan);
                            lastField = overlappedField;
                        }
                    }
                }
                XamDataGrid1.FieldLayouts[0].EnsureUniqueFieldPositions();
            }
        }

        private void XamDataGrid1_FieldPositionChanging(object sender, FieldPositionChangingEventArgs e)
        {
            if (e.ChangeReason == FieldPositionChangeReason.Moved)
            {
                // before the column is moved, record its old position
                var metadata = e.Field.Tag as FieldMetaData;
                metadata.OldColumn = e.Field.ActualPosition.Column;
            }
        }

        private void XamDataGrid1_FieldLayoutInitialized(object sender, FieldLayoutInitializedEventArgs e)
        {
            // give all the fields some metadata.  right now this is only used to track the old position of a column
            foreach (var field in e.FieldLayout.Fields)
                field.Tag = new FieldMetaData();
        }
    }

    public class FieldMetaData
    {
        public int OldColumn { get; set; }
    }
}